//+------------------------------------------------------------------+
//|                                                      monitor.mqh |
//|                               Copyright (c) 2010-2020, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+

#include <HashMapTemplate.mqh>
#include <crc64.mqh>

template<typename T>
union type2chars
{
  T x;
  uchar b[sizeof(T)];
  type2chars(T y): x(y) {}
};

class MultiSymbolMonitor
{
  protected:
    ENUM_TIMEFRAMES period;
    Map<string,datetime> lastTime;
    ulong lastCRC;

  public:
    MultiSymbolMonitor(): period(_Period), lastCRC(0) {}
    MultiSymbolMonitor(const ENUM_TIMEFRAMES p): period(p), lastCRC(0) {}
    
    void attach(const string symbol)
    {
      lastTime.put(symbol, NULL);
    }
    
    ulong refresh()
    {
      ulong crc = 0;
      for(int i = 0; i < lastTime.getSize(); i++)
      {
        string symbol = lastTime.getKey(i);
        datetime dt = iTime(symbol, period, 0);
        lastTime.put(symbol, dt);
        
        type2chars<datetime> t2c(dt);
        
        crc = crc64(crc, t2c.b, sizeof(datetime));
      }
      lastCRC = crc;
      return crc;
    }
    
    ulong getCRC() const
    {
      return lastCRC;
    }

    ulong check() const
    {
      ulong flags = 0;
      for(int i = 0; i < lastTime.getSize(); i++)
      {
        string symbol = lastTime.getKey(i);
        datetime dt = iTime(symbol, period, 0);
        
        if(dt != lastTime.get(symbol))
        {
          flags |= 1 << i;
        }
      }
      return flags;
    }
    
    void reset()
    {
      lastTime.reset();
      lastCRC = 0;
    }

};